package com.framsticks.framclipse.ui;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.internal.text.html.HTMLTextPresenter;
import org.eclipse.jface.text.DefaultInformationControl;
import org.eclipse.jface.text.IInformationControl;
import org.eclipse.jface.text.IInformationControlCreator;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextHover;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.Region;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.xtext.nodemodel.ICompositeNode;
import org.eclipse.xtext.nodemodel.ILeafNode;
import org.eclipse.xtext.nodemodel.util.NodeModelUtils;
import org.eclipse.xtext.resource.XtextResource;
import org.eclipse.xtext.ui.editor.XtextSourceViewerConfiguration;
import org.eclipse.xtext.ui.editor.model.IXtextDocument;
import org.eclipse.xtext.util.concurrent.IUnitOfWork;

import com.framsticks.framclipse.framScript.Function;
import com.framsticks.framclipse.framScript.Invocation;
import com.framsticks.framclipse.framScript.Proposable;
import com.framsticks.framclipse.framScript.QualifiedExpression;
import com.framsticks.framclipse.framScript.VariableDeclaration;
import com.framsticks.framclipse.framScript.VariableRef;
import com.framsticks.framclipse.ui.contentassist.DescriptorProvider;
import com.google.inject.Inject;

@SuppressWarnings("restriction")
public class FramScriptSourceViewerConfiguration extends XtextSourceViewerConfiguration {

	@Inject
	private DescriptorProvider descripter;

	@Override
	public IInformationControlCreator getInformationControlCreator(ISourceViewer sourceViewer) {
		return new IInformationControlCreator() {
			public IInformationControl createInformationControl(Shell parent) {
				return new DefaultInformationControl(parent, new HTMLTextPresenter(true));
			}
		};
	}

	@Override
	public ITextHover getTextHover(final ISourceViewer sourceViewer, String contentType) {
		return new ITextHover() {

			@Override
			public IRegion getHoverRegion(ITextViewer textViewer, int offset) {
				return new Region(offset, 0);
			}

			@Override
			public String getHoverInfo(ITextViewer textViewer, IRegion hoverRegion) {
				final int offset = hoverRegion.getOffset();
				final IXtextDocument model = (IXtextDocument) sourceViewer.getDocument();
				return model.readOnly(new IUnitOfWork<String, XtextResource>() {

					@Override
					public String exec(XtextResource state) throws Exception {
						ICompositeNode root = state.getParseResult().getRootNode();
//						AbstractNode current = ParseTreeUtil.getCurrentOrFollowingNodeByOffset(root, offset);
						ILeafNode current = NodeModelUtils.findLeafNodeAtOffset(root, offset);
//						EObject semantic = NodeUtil.getNearestSemanticObject(current);
						EObject semantic = current.getSemanticElement();

						if (semantic instanceof QualifiedExpression) {
							return getMessage((QualifiedExpression) semantic);
						}
						return null;
					}
				});
			}
		};
	}

	private String getMessage(QualifiedExpression expression) {
		if (expression instanceof Invocation) {
			Function function = ((Invocation) expression).getFunction();
			if (function instanceof Proposable) {
				return descripter.description(((Proposable) function).description());
			}
		} else if (expression instanceof VariableRef) {
			VariableDeclaration var = ((VariableRef) expression).getVar();
			if (var instanceof Proposable) {
				return descripter.description(((Proposable) var).description());
			}
		}
		return null;
	}

}
